//
//  CCHDRProcessingFacade.h
//  ITCCHDR
//
//  Created by Jens Daemgen on 7/24/14.
//  Copyright (c) 2014 Cocologics. All rights reserved.
//

@import UIKit;
#import <ITCCHDR/ITCCCommonUtils.h>
#import <ITCCHDR/hdr_interface.h>

@class CCYUVImage;
@protocol CCHDRProcessingFacadeDelegate;

@interface CCHDRProcessingFacade : NSObject

+ (CCHDRProcessingFacade *)sharedInstance;

/**
 Initialize buffers with the given settings and images, and process the selected HDR preset.
 
 The blockOfMemory can either be nil, then a temporary buffer will be allocated and freed internally,
 or it can be set to a region of memory that is expected to contiguously hold the planes of all input images.
 In the latter case, the images may be reordered and the memory will be parly used as temporary buffer while processing.
 
 Returns an NSArray with [autoImage, hdrImage];
 */
- (NSArray<CCYUVImage*> *)processHDRImageWithYUVImages:(NSArray <CCYUVImage *>*)yuvImages
                                     size:(CGSize)size
                           referenceImage:(int)referenceImage
                                   preset:(HDR_PRSET_SELECTION_T)preset
                         imageOrientation:(UIImageOrientation)imageOrientation
                              aspectRatio:(float)aspectRatio
                             croppingRect:(CGRect *)croppingRect
                            blockOfMemory:(NSData *)blockOfMemory
                                autoImage:(CCYUVImage *)autoImage
                                    error:(NSError **)error;

- (CCYUVImage *)processHDRImageWithToneMappingPreset:(HDR_PRSET_SELECTION_T)preset;

- (void)deleteHDRInstance;

- (CCYUVImage *)croppedAutoImage;

- (UIImage *)cropAndConvertYUVImage:(CCYUVImage *)yuvImage
                      toAspectRatio:(float)aspectRatio
                 withinCroppingRect:(CGRect)croppingRect
                   imageOrientation:(UIImageOrientation)imageOrientation;

@property (nonatomic, weak) id<CCHDRProcessingFacadeDelegate> delegate;
@property (nonatomic, assign) HDR_CAMERA_MODEL_T cameraModel;
@property (nonatomic, assign) uint32_t numCores;
@property (nonatomic, assign) HDR_IP_SELECTION_T hdrIPMode;

@end

@protocol CCHDRProcessingFacadeDelegate <NSObject>

- (void)hdrProcessingFacade:(CCHDRProcessingFacade *)processingFacade didUpdateProgress:(float)progress;

@end
