//
//  CCYUVImage.h
//  ITCCHDR
//
//  Created by Jens Daemgen on 7/29/14.
//  Copyright (c) 2014 Cocologics. All rights reserved.
//

@import UIKit;
@import CoreMedia;

@interface CCYUVImage : NSObject

@property (nonatomic, assign) CGSize size;
@property (nonatomic, assign) size_t bytesPerRowOfLumaBuffer;
@property (nonatomic, strong) NSData *ypData;

@property (nonatomic, assign) size_t bytesPerRowOfChromaBuffers;
// YUV420sp
@property (nonatomic, strong) NSData *cbCrData;
// YUV420p
@property (nonatomic, strong) NSData *cbData;
@property (nonatomic, strong) NSData *crData;

@property (nonatomic, assign) UIImageOrientation imageOrientation;
@property (nonatomic, assign) float exposureTime;
@property (nonatomic, assign) float ISO;
@property (nonatomic, strong) NSDictionary *metadataDictionary;

/**
 * This method will copy the bytes.
 */
+ (CCYUVImage *)yuvImageWithYuvImage:(CCYUVImage *)yuvImage;

+ (CCYUVImage *)yuvImageWithData:(NSData *)yuvData size:(CGSize)size;

/**
 * This method does _not_ copy the bytes. Stride is assumed to be equal to width.
 */
+ (CCYUVImage *)yuvImageNoCopyWithYpBytes:(Byte *)ypBytes
                                  cbBytes:(Byte *)cbBytes
                                  crBytes:(Byte *)crBytes
                                 withSize:(CGSize)size
                         imageOrientation:(UIImageOrientation)imageOrientation
                             freeWhenDone:(BOOL)freeWhenDone;

/**
 * This method will copy the bytes
 */
+ (CCYUVImage *)yuvImageWithYpBytes:(Byte *)ypBytes
                          cbCrBytes:(Byte *)cbCrBytes
                           withSize:(CGSize)size
            bytesPerRowOfLumaBuffer:(size_t)bytesPerRowOfLumaBuffer
          bytesPerRowOfChromaBuffer:(size_t)bytesPerRowOfChromaBuffer
                   imageOrientation:(UIImageOrientation)imageOrientation;

/**
 * This method will _not_ copy the bytes
 */
+ (CCYUVImage *)yuv420spImageNoCopyWithYpBytes:(Byte *)ypBytes
                       bytesPerRowOfLumaBuffer:(size_t)bytesPerRowOfLumaBuffer
                                     cbCrBytes:(Byte *)cbCrBytes
                     bytesPerRowOfChromaBuffer:(size_t)bytesPerRowOfChromaBuffer
                                      withSize:(CGSize)size
                              imageOrientation:(UIImageOrientation)imageOrientation
                                  freeWhenDone:(BOOL)freeWhenDone;


/**
 * This method will copy the bytes. Stride is assumed to be equal to width.
 */
+ (CCYUVImage *)yuvImageWithYpBytes:(Byte *)ypBytes
                          cbCrBytes:(Byte *)cbCrBytes
                           withSize:(CGSize)size
                   imageOrientation:(UIImageOrientation)imageOrientation;

/**
 * This method will _not_ copy the bytes, but only construct a CCYUVImage with ypData pointing to ypBytes. Stride is assumed to be equal to width.
 */
+ (CCYUVImage *)lumaBufferNoCopyWithYpBytes:(Byte *)ypBytes
                                   withSize:(CGSize)size
                           imageOrientation:(UIImageOrientation)imageOrientation
                               freeWhenDone:(BOOL)freeWhenDone;


- (CCYUVImage *)previewYUVImageWithSize:(CGSize)size;



///// Not toll-free bridged API of CCYUVImage

/**
 * This method will copy the bytes and de-interleave the cbCrBytes.
 */
+ (CCYUVImage *)yuv420pImageWithYpBytes:(Byte *)ypBytes
                              cbCrBytes:(Byte *)cbCrBytes
                               withSize:(CGSize)size
                bytesPerRowOfLumaBuffer:(size_t)bytesPerRowOfLumaBuffer
              bytesPerRowOfChromaBuffer:(size_t)bytesPerRowOfChromaBuffer
                       imageOrientation:(UIImageOrientation)imageOrientation; // converts YUV420sp to YUV420p


- (void)writeDataToPath:(NSString *)filePath;
- (void)saveDataToDocumentsDirectoryWithName:(NSString *)fileName;

@end
