//  Created by Jens Daemgen on 8/25/14.
//  Copyright (c) 2014 Cocologics. All rights reserved.
//

@import UIKit;
#import <ITCCHDR/CCYUVImage.h>
#import <ITCCHDR/ITCCCommonUtils.h>
#import <ITCCHDR/seb_interface.h>

@class ITCCExposureSettings;
@class ITCCExposureBracketSettings;

@interface ITCCHDRSceneAnalyzer: NSObject

/**
 @return NO in case setup failed
 */
- (BOOL)setSize:(CGSize)size
    bytesPerRow:(size_t)bytesPerRow
         minISO:(float)minISO
         maxISO:(float)maxISO
minExposureTime:(float)minExposureTime
maxExposureTime:(float)maxExposureTime
      sebIPMode:(SEB_IP_SELECTION_T)sebIPMode
     sebOutMode:(SEB_OUT_MODE_T)sebOutMode;

/**
 * Get an initial estimate of the bracket settings by passing an auto exposed frame (of type CCYUVImage)
 */
- (ITCCExposureBracketSettings *)estimatedExposureBracketSettingsForAutoExposedFrame:(CCYUVImage *)yuvImageAutoExposed;

/**
 * Get the final exposure bracket settings. Pass three frames (<CCYUVImage>) in the following order: dark, medium, bright
 */
- (ITCCExposureBracketSettings *)refinedExposureBracketSettingsForFrames:(NSArray *)yuvImages;

/**
 * Get the estimate of bracket settings for the next frame using the data of the current image and the next auto exposed frame
 */
- (ITCCExposureBracketSettings *)estimateExposureSettingsForNextAutoExposedFrame:(CCYUVImage *)yuvImageAutoExposed;

@property (nonatomic, assign) SEB_CAMERA_MODEL_T cameraModel;
@property (nonatomic, readonly) BOOL initialized;
@property (nonatomic, assign) SEB_IP_SELECTION_T sebIPMode;
@property (nonatomic, assign) SEB_OUT_MODE_T sebOutMode;
@property (nonatomic, assign) SEB_OUT_MODE_T recommendedSebOutMode;
@property (nonatomic, assign) SEB_OUT_MODE_T maxNumOfSettingsReqd;
@property (nonatomic, assign) float maxExposureTime;
@property (nonatomic, assign) BOOL timelapseMode;

@end

@interface ITCCExposureSettings : NSObject

+ (instancetype)exposureSettingsWithISO:(float)ISO exposureTime:(float)exposureTime;

@property (nonatomic, assign) float ISO;
@property (nonatomic, assign) float exposureTime;
@property (nonatomic, readonly) float exposureValue;

// The SEB library requests images that are far too bright iPhones XS, XSMax, XR.
// In that case we "sanitize" the requested exposures by adding a new exposure in the middle.
// This flag indicates whether it was added by our sanitizer or by SEB directly.
@property (nonatomic, readonly) BOOL wasAddedBySanitizer;

// Indicates whether these exposure settings are equal to those suggested by Apple as autoExposure
@property (nonatomic, assign) BOOL isAutoImage;

@end


@interface ITCCExposureBracketSettings : NSObject

@property (nonatomic, strong) NSArray <ITCCExposureSettings *> *imageSettingsArray;
@property (nonatomic, strong) ITCCExposureSettings *autoImageSettings;
@property (nonatomic, assign) float maxISO;
@property (nonatomic, assign) float maxExposureTime;

@end
