/*
 * hdr_interface.h
 *
 *  Created on: Jan 24, 2012
 *      Author: 100354
 */
/*****************************************************************************/
/*                                                                           */
/*                           HDR Imaging Application                         */
/*                     ITTIAM SYSTEMS PVT LTD, BANGALORE                     */
/*                             COPYRIGHT(C) 2012                             */
/*                                                                           */
/*  This program  is  proprietary to  Ittiam  Systems  Private  Limited  and */
/*  is protected under Indian  Copyright Law as an unpublished work. Its use */
/*  and  disclosure  is  limited by  the terms  and  conditions of a license */
/*  agreement. It may not be copied or otherwise  reproduced or disclosed to */
/*  persons outside the licensee's organization except in accordance with the*/
/*  terms  and  conditions   of  such  an  agreement.  All  copies  and      */
/*  reproductions shall be the property of Ittiam Systems Private Limited and*/
/*  must bear this notice in its entirety.                                   */
/*                                                                           */
/*****************************************************************************/
#ifndef HDR_INTERFACE_H_
#define HDR_INTERFACE_H_
#ifdef __cplusplus
extern "C" {
#endif
/*****************************************************************************/
/* Constant Definitions                                                      */
/*****************************************************************************/
#define HDR_MAX_NUM_IMAGE_COMPONENTS 3
#define HDR_MAX_NUM_IMAGES			 10
#define HDR_MAX_NUM_IMAGES_OLD			 4
//#define HDR_EXT_METADATA_SIZE		8192
#define HDR_EXT_METADATA_SIZE		16384
#define HDR_NUM_IMAGES_SUPPORTED	3
#define HDR_NUM_REF_SUPPORTED		1
#define TOTAL_PIX_RANGE				256
#define VERSION_STRING_SIZE			32
#define REF_VERSION_REMAP_RADIANCE  580 /* reference version is 5.8 */
#define NUM_AFFINE_PARAMS			6
#define NUM_MATCH_HIST_POINTS		16


/*****************************************************************************/
/* Enumeration definitions                                                   */
/*****************************************************************************/
typedef enum {
	/*0*/HDR_NO_ERROR =  0,
	/*1*/HDR_ERROR_NULL_PARAMS,
	/*2*/HDR_ERROR_NULL_CALLBACKS,
	/*3*/HDR_ERROR_MEM_RECORD,
	/*4*/HDR_ERROR_MEM_ALLOCATE,
	/*5*/HDR_ERROR_ALGO_INIT,
	/*6*/HDR_PROCESS_ERROR,
	/*7*/HDR_NULL_HANDLE,
	/*8*/HDR_NULL_IN_PARAMS,
	/*9*/HDR_NULL_OUT_PARAMS,
	/*10*/HDR_NULL_IN_BUFFERS,
	/*11*/HDR_NULL_OUT_BUFFERS,
	/*12*/HDR_INVALID_IN_PARAMS,
	/*13*/HDR_INVALID_IN_SIZES,
	/*14*/HDR_INVALID_OUT_SIZES,
	/*15*/HDR_INVALID_EXPOSURE,
	/*16*/HDR_OUT_OF_BOUND_EXP_RATIO,
	/*17*/HDR_FAILED_MEM_FREE,
	/*18*/HDR_CORRUPT_NUM_MEM_RECORDS,
	/*19*/HDR_FEATURE_NOT_SUPPORTED,
	/*20*/HDR_UNKNOWN_IP_ENUM
} HDR_ERROR_TYPE_T ;
typedef enum HDR_IMAGE_FORMAT_T
{
	ITT_HDR_JPG = 0,
	ITT_HDR_YUV420 = 1
}HDR_IMAGE_FORMAT_T;

typedef enum HDR_PROCESS_STATUS_T
{
	HDR_FAIL = -1,
	HDR_SUCCESS = 0,

}HDR_PROCESS_STATUS_T;

typedef enum HDR_CAMERA_MODEL_T
{
	HDR_LG_HUB = 0,
	HDR_NEXUS_PRIME,
	HDR_DROID_3,
	HDR_NEXUS_S,
	HDR_QUALCOM_MDP,
	HDR_QUALCOM_8x25,
	HDR_SAMSUNG_TAB_3,
	HDR_SAMSUNG_S4,
	HDR_LG_NEXUS_4,
	HDR_LG_NEXUS_5,
	HDR_MOTO_NEXUS_6,
	HDR_MICROMAX_CANVAS,
	HDR_XPERIA_Z_ULTRA,
	HDR_MOTO_X,
	HDR_MODEL_UNKNOWN,


	HDR_IPOD_4G = 100,
	HDR_IPOD_5G,
	HDR_IPOD_6G,
	HDR_IPAD_2,
	HDR_IPAD_3,
	HDR_IPAD_4,
	HDR_IPAD_MINI,
	HDR_IPAD_MINI_4,
	HDR_IPAD_AIR,
	HDR_IPAD_AIR_2,
	HDR_IPAD_PRO,
	HDR_IPHONE_3GS,
	HDR_IPHONE_4,
	HDR_IPHONE_4S,
	HDR_IPHONE_5,
	HDR_IPHONE_5S,
	HDR_IPHONE_6,
	HDR_IPHONE_6_Plus,
	HDR_IPHONE_6S,
	HDR_IPHONE_6S_Plus,
	HDR_IPHONE_7,
	HDR_IPHONE_7_Plus,
	HDR_IOS_MODEL_UNKNOWN,

	HDR_INTEL_BAYTRAIL_MDP = 200,
	HDR_INTEL_MODEL_UNKNOWN,

	HDR_NOKIA_LUMIA_630 = 300,
	HDR_NOKIA_LUMIA_920,
	HDR_NOKIA_LUMIA_1020,
	HDR_NOKIA_LUMIA_1520,
	HDR_NOKIA_LUMIA_530,

	HDR_CALIBRATE_MODEL = 400
}HDR_CAMERA_MODEL_T;

typedef enum HDR_IP_SELECTION_T
{
	HDR_GET_HDR = 0,
	HDR_GET_LOW_LIGHT,
	HDR_TONE_MAP,
	HDR_GET_RADIANCE,
	HDR_MODE_UNKNOWN
}HDR_IP_SELECTION_T;

typedef enum HDR_PRSET_SELECTION_T
{
	HDR_NATURAL = 0,
	HDR_LIVELY,
	HDR_DRAMATIC,
	HDR_BLEACH,
	HDR_BW_ART,
	HDR_TONE_MAP_EDIT_MODE
}HDR_PRSET_SELECTION_T;

/*****************************************************************************/
/* Data structure definitions                                                */
/*****************************************************************************/
typedef struct hdr_init_params_t
{
	WORD32	i4_max_width;
	WORD32	i4_max_height;
	WORD32	i4_max_stride;
	WORD32	i4_max_num_images;
	WORD32	i4_image_format;
	WORD32	i4_camera_phone;
	WORD32  i4_mode;
	WORD32  i4_num_cores;
	WORD32	ai4_exposure_time[HDR_MAX_NUM_IMAGES];
	WORD32	i4_memory_available;
}hdr_init_params_t;

typedef struct	hdr_tone_edit_params_t
{
	WORD32 i4_brightness;
	WORD32 i4_saturation;			/*Range [-100 100]*/
	WORD32 i4_contrast;
	WORD32 i4_warmth;
	WORD32 i4_tint;
	WORD32 i4_highlights;			/*Range [-100 100]*/
	WORD32 i4_shadows;				/*Range [-100 100]*/
	WORD32 i4_strength;
	WORD32 i4_white_point;
	WORD32 i4_black_point;
	WORD32 i4_preset_enum; /* HDR_PRSET_SELECTION_T */
	WORD32 i4_tone_mapping_scale; 	/*Range : [0 100]*/
	//float 	f_tonemapping_scale;
}hdr_tone_edit_params_t;

typedef struct	hdr_in_params_t
{
	WORD32	i4_num_images;
	WORD32	i4_ref_image_num;
	WORD32 	i4_width;
	WORD32 	i4_height;
	WORD32 	i4_stride;
	WORD32	i4_format;
	hdr_tone_edit_params_t s_tone_edit_params;

}hdr_in_params_t;

typedef struct	tonemap_in_params_t
{
	WORD32 	i4_width;
	WORD32 	i4_height;
	WORD32 	i4_stride;
	hdr_tone_edit_params_t s_tone_edit_params;

}tonemap_in_params_t;


typedef struct	hdr_in_image_t
{
	WORD32	i4_num_buffers;
	UWORD8	*pu1_bufs[HDR_MAX_NUM_IMAGE_COMPONENTS];
	WORD32	i4_buffer_size[HDR_MAX_NUM_IMAGE_COMPONENTS];
	WORD32	i4_exposure_time;
}hdr_in_image_t;

typedef struct	hdr_in_buffers_t
{
	hdr_in_image_t	as_in_image[HDR_MAX_NUM_IMAGES];
}hdr_in_buffers_t;


typedef struct  tonemap_in_buffers_t
{
	UWORD16  *pu2_out_radiance_buf;
	WORD32   i4_radiance_size;
}tonemap_in_buffers_t;

typedef struct hdr_metadata_t
{
	WORD32  ai4_expsure_time [HDR_MAX_NUM_IMAGES];
	WORD32  i4_image_width;
	WORD32  i4_image_height;
	WORD32  i4_image_stride;
	UWORD8  au1_extended_metadata [HDR_EXT_METADATA_SIZE];
} hdr_metadata_t;

typedef struct hdr_out_buffers_t
{
	WORD32  i4_num_buffers_yuv;
	UWORD8  *pu1_out_yuv_buf [HDR_MAX_NUM_IMAGE_COMPONENTS];
	WORD32   ai4_yuv_size[HDR_MAX_NUM_IMAGE_COMPONENTS];
	UWORD16	*pu2_out_radiance_buf;
	WORD32 i4_radiance_size;
	hdr_metadata_t *ps_hdr_metadata;
} hdr_out_buffers_t;

typedef struct hdr_out_params_t
{
	WORD32  i4_num_buffers_yuv;
	UWORD8	*pu1_out_hdr[HDR_MAX_NUM_IMAGE_COMPONENTS];
	UWORD16	*pu2_out_radiance;
	WORD32 i4_out_width;
	WORD32 i4_out_height;
	WORD32 i4_out_stride;
	WORD32	i4_error_code;

}hdr_out_params_t;

typedef struct tonemap_in_out_buffers_t
{
	WORD32  i4_num_buffers_yuv;
	UWORD8  *pu1_yuv_buf [HDR_MAX_NUM_IMAGE_COMPONENTS];
	WORD32   ai4_yuv_size [HDR_MAX_NUM_IMAGE_COMPONENTS];
	hdr_metadata_t *ps_hdr_metadata;
} tonemap_in_out_buffers_t;

typedef struct tonemap_out_params_t
{
	WORD32  i4_num_buffers_yuv;
	UWORD8  *pu1_out_hdr[HDR_MAX_NUM_IMAGE_COMPONENTS];
	WORD32  i4_out_width;
	WORD32  i4_out_height;
	WORD32  i4_out_stride;
	WORD32  i4_error_code;
} tonemap_out_params_t;


typedef struct hdr_callbacks_t
{
	void * (*hdr_memory_allocate)(WORD32 i4_size);
	void   (*hdr_memory_free)(void *pv_ptr);

	WORD32   (*hdr_file_dump)(void *pv_ptr,
 				 WORD32 i4_size,
          		char *pi1_file_name,
          		WORD32 is_append_mode);
	UWORD32 (*hdr_thread_handle_size)(void);
	WORD32 (*hdr_thread_create)(
			void *thread_handle,
			void *attribute,
			void *strt,
			void *argument);
	WORD32 (*hdr_thread_join)(
			void *thread_handle,
			void ** val_ptr);
#ifdef IOS
	void (*hdr_update_progress)(WORD32 i4_progress);
	WORD32 (*hdr_get_continue_command)(void);
	void (*hdr_free_non_ref_bufs)(void);
#endif

}hdr_callbacks_t;

/*****************************************************************************/
/* Function prototypes														 */
/*****************************************************************************/
void* 	hdr_init (  hdr_init_params_t *ps_params,
		hdr_callbacks_t 	*ps_callbacks,
		WORD32 		*pi4_error_code);


WORD32 hdr_process (void  	*pv_handle,
		hdr_in_params_t	    *ps_in_params,
		hdr_out_params_t    *ps_out_params,
		hdr_in_buffers_t 	*ps_in_image,
		hdr_out_buffers_t     *ps_out_image);

WORD32 hdr_tone_map_reset(WORD32 ai4_exposure_time[HDR_MAX_NUM_IMAGES],
						  void 			 	 	 *pv_handle);


WORD32 hdr_tonemap_process(void *pv_handle,
		tonemap_in_params_t *ps_in_params,
		tonemap_out_params_t *ps_out_params,
		tonemap_in_buffers_t *ps_in_buffers,
		tonemap_in_out_buffers_t *ps_in_out_buffers);

WORD32  hdr_delete( void 	*pv_handle);

WORD32 hdr_get_desired_ev (UWORD8 *pu1_review_buf,
		WORD32 i4_height,
		WORD32 i4_width,
		WORD32 i4_stride,
		UWORD32 *pi4_ref_histogram,
		UWORD32 *pi4_cur_histogram,
		WORD32 i4_current_EV,
		WORD32 i4_camera_model,
		WORD32 i4_num_steps_in_EV,
		WORD32 i4_maxEv,
		WORD32 i4_minEv,
		WORD32 *pi4_observed_EV);

WORD32 hdr_warp(void *pv_handle, hdr_in_image_t *ps_in_image, hdr_in_image_t *ps_ref_image,
		hdr_in_image_t *ps_out_image);

WORD32 hdr_free_resources(void *pv_handle);

#ifdef __cplusplus
} /* closing brace for extern "C" */
#endif

#endif /* HDR_INTERFACE_H_ */
