#ifndef _ITT_LL_INTERFACE_H_
#define _ITT_LL_INTERFACE_H_

#define ITT_LL_NUM_AFFINE_PARAMS			  6
#define ITT_LL_MAX_NUM_IMAGE_COMPONENTS 	  3
#define ITT_LL_MAX_NUM_IMAGES_MFLE			256
#define ITT_LL_MAX_NUM_IMAGES_MFNR			 64



typedef enum ITT_LL_IMAGE_FORMAT_T
{
	ITT_LL_YUV420P = 0,
	ITT_LL_YUV420SP = 1,
} ITT_LL_IMAGE_FORMAT_T;

typedef enum ITT_LL_PROCESS_STATUS_T
{
	ITT_LL_FAIL = -1,
	ITT_LL_SUCCESS = 0,
} ITT_LL_PROCESS_STATUS_T;

typedef enum ITT_LL_CAMERA_MODEL_T
{
	ITT_LL_CURVE_INPUT = 0,
	ITT_LL_IPAD_2,
	ITT_LL_IPAD_3,
	ITT_LL_IPAD_4,
	ITT_LL_IPAD_MINI,
	ITT_LL_IPAD_MINI4,
	ITT_LL_IPAD_AIR,
	ITT_LL_IPAD_AIR_2,
	ITT_LL_IPAD_PRO,
	ITT_LL_IPHONE_5,
	ITT_LL_IPHONE_5S,
	ITT_LL_IPHONE_6,
	ITT_LL_IPHONE_6_Plus,
	ITT_LL_IPHONE_6S,
	ITT_LL_IPHONE_6S_Plus,
	ITT_LL_IPOD_5G,
	ITT_LL_IPOD_6G,
	ITT_LL_MODEL_UNKNOWN
}ITT_LL_CAMERA_MODEL_T;


typedef enum ITT_LL_IP_SELECTION_T
{
	ITT_LL_GET_MFNR = 0,
	ITT_LL_GET_MFLE,
	ITT_LL_MODE_UNKNOWN
}ITT_LL_IP_SELECTION_T;

typedef enum ITT_LL_POST_WB_CORRECTION_T
{
	ITT_LL_WB_CORRECTION_NOT_REQUIRED = 0,
	ITT_LL_WB_CORRECTION_REQUIRED

}ITT_LL_POST_WB_CORRECTION_T;


typedef enum {
	/*0*/ITT_LL_NO_ERR =  0,
	/*1*/ITT_LL_ERR_NULL_PARAMS,
	/*2*/ITT_LL_ERR_NULL_CALLBACKS,
	/*3*/ITT_LL_ERR_MEM_ALLOCATE,
	/*4*/ITT_LL_ERR_ALGO_INIT,
	/*5*/ITT_LL_ERR_UNKNOWN_IP_ENUM,
	/*6*/ITT_LL_ERR_UNSUPPORTED_IMG_FORMAT,
	/*7*/ITT_LL_ERR_GPU_INIT,
	/*8*/ITT_LL_ERR_IMG_BUF_ALLOC,
	/*9*/ITT_LL_ERR_THREAD_CREATE,
	/*10*/ITT_LL_ERR_MAX_NUM_IMAGES_EXCEEDED,
	/*11*/ITT_LL_ERR_NULL_HANDLE,
	/*12*/ITT_LL_ERR_NULL_IN_PARAMS,
	/*13*/ITT_LL_ERR_NULL_OUT_PARAMS,
	/*14*/ITT_LL_ERR_NULL_IN_BUFFERS,
	/*15*/ITT_LL_ERR_NULL_OUT_BUFFERS,
	/*16*/ITT_LL_ERR_INVALID_IN_PARAMS,
	/*17*/ITT_LL_ERR_NUM_IMAGES_EXCEEDED,
	/*18*/ITT_LL_ERR_BUFFER_Q_FULL,
	/*19*/ITT_LL_ERR_NO_IMGS_TO_PROCESS,
	/*20*/ITT_LL_ERR_INVALID_OUT_SIZES,

  CC_LL_ERR_CUSTOM = 2110, // Used for custom error reporting
} ITT_LL_ERR_TYPE_T ;


typedef struct _itt_ll_init_params
{
	WORD32	i4_width;
	WORD32	i4_height;
	WORD32	i4_max_num_images;
    WORD32  i4_preview_width;
    WORD32  i4_preview_height;
	WORD32	i4_image_format;
	WORD32	i4_camera_phone;
	WORD32  i4_algo_mode;
	WORD32  i4_post_enhance_flag;
	WORD32	i4_auto_ISO_val;
	WORD32	i4_auto_exp_time_in_us;
	void	*ps_prev_metadata;
	WORD32	i4_gpu_available;
	WORD32  i4_num_cores;
} itt_ll_init_params_t;

/*Input parmaeters to itt_ll_add_image*/
typedef struct itt_ll_in_params
{
      WORD32 i4_ISO_val;
      WORD32 i4_exp_time_in_us;
} itt_ll_in_params_t;

#ifndef IOS
typedef struct _itt_ll_img_buf
{
	WORD32 i4_num_buffers;
	UWORD8 *pu1_buf[ITT_LL_MAX_NUM_IMAGE_COMPONENTS];
	WORD32 ai4_buf_size[ITT_LL_MAX_NUM_IMAGE_COMPONENTS];
	WORD32 ai4_stride[ITT_LL_MAX_NUM_IMAGE_COMPONENTS];

} itt_ll_img_buf_t;
#endif

typedef struct _itt_ll_out_params
{
	WORD32	i4_num_buffers;
	UWORD8 *pu1_out_lowlight[ITT_LL_MAX_NUM_IMAGE_COMPONENTS];
	WORD32	i4_out_width;
	WORD32 	i4_out_height;
	float  f_out_scale;

}itt_ll_out_params_t;


typedef struct _itt_ll_callbacks
{
	void*  (*itt_ll_memory_allocate)(WORD32 i4_size);

	void   (*itt_ll_memory_free)(void *pv_ptr);

	void*  (*itt_ll_img_buffer_allocate)(WORD32 i4_width,
										 WORD32 i4_height,
										 WORD32 i4_image_format,
										 void	 **ppv_app_cntxt);

	WORD32 (*itt_ll_release_input_image)(void	*pv_in_buf_cntxt,
										 WORD32  preview_available);

	WORD32 (*itt_ll_output_image)(void 				  *pv_out_buf_cntxt,
								  itt_ll_out_params_t *ps_out_params);
}itt_ll_callbacks_t;


/* API Function Declarations */

void*   itt_ll_init(itt_ll_init_params_t *ps_params,
					itt_ll_callbacks_t   *ps_callbacks,
					WORD32				 *pi4_error_code);

WORD32  itt_ll_add_image(void 			    *pv_handle,
						 void			    *ps_image,
						 void 			    *pv_in_buf_cntxt,
						 itt_ll_in_params_t *ps_in_params);

WORD32  itt_ll_process(void 			*pv_handle,
					   void 			*ps_out_image,
					   void 			*pv_out_buf_cntxt,
					   float            f_in_scale,
					   WORD32           i4_final_output);

WORD32  itt_ll_delete(void *pv_handle);


/* Function table pointer */
typedef struct _ITT_LOWLIGHT_API
{
	void*   (*itt_ll_init)(itt_ll_init_params_t *ps_params,
						   itt_ll_callbacks_t   *ps_callbacks,
						   WORD32				*pi4_error_code);

	WORD32  (*itt_ll_add_image)(void  			   *pv_handle,
							    void			   *ps_image,
							    void 			   *pv_in_buf_cntxt,
							    itt_ll_in_params_t *ps_in_params);

	WORD32  (*itt_ll_process)(void 			   *pv_handle,
							  void			   *ps_out_image,
							  void 			   *pv_out_buf_cntxt,
							  float            f_in_scale,
							  WORD32           i4_final_output);

	WORD32  (*itt_ll_delete) (void *pv_handle);
} ITT_LOWLIGHT_API_t;

extern ITT_LOWLIGHT_API_t ITTIAM_LOWLIGHT_API;

#endif /*_ITT_LL_INTERFACE_H_*/
