/*
 * seb_interface.h
 *
 *  Created on: Oct 05, 2012
 *      Author: 100457
 */
/*****************************************************************************/
/*                                                                           */
/*                           SEB Imaging Application                         */
/*                     ITTIAM SYSTEMS PVT LTD, BANGALORE                     */
/*                             COPYRIGHT(C) 2012                             */
/*                                                                           */
/*  This program  is  proprietary to  Ittiam  Systems  Private  Limited  and */
/*  is protected under Indian  Copyright Law as an unpublished work. Its use */
/*  and  disclosure  is  limited by  the terms  and  conditions of a license */
/*  agreement. It may not be copied or otherwise  reproduced or disclosed to */
/*  persons outside the licensee's organization except in accordance with the*/
/*  terms  and  conditions   of  such  an  agreement.  All  copies  and      */
/*  reproductions shall be the property of Ittiam Systems Private Limited and*/
/*  must bear this notice in its entirety.                                   */
/*                                                                           */
/*****************************************************************************/
#ifndef SEB_INTERFACE_H_
#define SEB_INTERFACE_H_

/*****************************************************************************/
/* Constant Definitions                                                      */
/*****************************************************************************/
#define SEB_MAX_NUM_IMAGES			 3
#define SEB_MAX_NUM_IMAGE_COMPONENTS 3		/* indicating YUV components number */
#define TOTAL_PIX_RANGE				 256	/* indicating Total pixel range 	*/
#define SEB_MAX_NUM_PTS				 2
#define SEB_MAX_NUM_BRACKETS		 5

/*****************************************************************************/
/* Enumeration definitions                                                   */
/*****************************************************************************/
typedef enum {			/* Error Enumerations */
	SEB_NO_ERROR		 =  0,
	SEB_ERROR_NULL_PARAMS,
	SEB_ERROR_NULL_CALLBACKS,
	SEB_ERROR_MEM_RECORD,
	SEB_ERROR_MEM_ALLOCATE,
	SEB_ERROR_ALGO_INIT,
	SEB_PROCESS_ERROR,
	SEB_NULL_HANDLE,
	SEB_NULL_IN_PARAMS,
	SEB_INVALID_IN_PARAMS,
	SEB_NULL_OUT_PARAMS,
	SEB_NULL_IN_BUFFERS,
	SEB_FAILED_MEM_FREE,
	SEB_CORRUPT_NUM_MEM_RECORDS,
	SEB_BRIGHT_POINT_EXCEEDED_1,
	SEB_BRIGHT_POINT_EXCEEDED_0,
	SEB_DARK_POINT_EXCEEDED_1,
	SEB_DARK_POINT_EXCEEDED_0,
	SEB_BRIGHT_PROJECTION_FAILED,
	SEB_DARK_PROJECTION_FAILED
}SEB_ERROR_TYPE_T ;

typedef enum SEB_IMAGE_FORMAT_T
{
	ITT_SEB_JPG = 0,		/* Not yet supported by IP */
	ITT_SEB_YUV420 = 1		/* Current mode supported */
}SEB_IMAGE_FORMAT_T;

typedef enum SEB_PROCESS_STATUS_T
{
	SEB_FAIL = -1,			/* Process Failure */
	SEB_SUCCESS = 0,		/* Process Success */
	SEB_INCORRECT_IMAGE = 1/* Dummy value Not used so far */

}SEB_PROCESS_STATUS_T;

typedef enum SEB_CAMERA_MODEL_T
{
	SEB_IPOD_4G = 0,/* Default Phone camera model */
	SEB_IPOD_5G,
	SEB_IPOD_6G,
	SEB_IPAD_2,
	SEB_IPAD_3,
	SEB_IPAD_4,
	SEB_IPAD_MINI,
	SEB_IPAD_MINI_4,
	SEB_IPAD_AIR,
	SEB_IPAD_AIR_2,
	SEB_IPAD_PRO,
	SEB_IPHONE_3GS,
	SEB_IPHONE_4,
	SEB_IPHONE_4S,
	SEB_IPHONE_5,
	SEB_IPHONE_5S,
	SEB_IPHONE_6,
	SEB_IPHONE_6_Plus,
	SEB_IPHONE_6S,
	SEB_IPHONE_6S_Plus,
	SEB_IPHONE_7,
	SEB_IPHONE_7_Plus,
    SEB_IPHONE_8,
    SEB_IPHONE_8_Plus,
    SEB_IPHONE_X,

	SEB_LG_NEXUS_5 = 100,

	SEB_MODEL_UNKNOWN = 500
}SEB_CAMERA_MODEL_T;

typedef enum SEB_IP_SELECTION_T
{
	SEB_GET_METERING_POINT = 0,		/* Default Call type */
	SEB_GET_EXPOSURE_ISO,
	SEB_LOW_LIGHT_MODE,
	SEB_MODE_UNKNOWN		/* Not Used yet		 */
}SEB_IP_SELECTION_T;

typedef enum SEB_OUT_MODE_T         /* This enum is used to select the number of images
                                        (dark + bright + ref) to be given as output*/
{
    SEB_AUTO_SELECT_MODE	 = 0,       /* Algorithm chooses between 3 and 5 images */
    SEB_GET_3_OUT_IMAGES	 = 3,       /* Algorithm gives out 3 images */
    SEB_GET_4_OUT_IMAGES     = 4,       /* Algorithm gives out 4 images */
    SEB_GET_5_OUT_IMAGES	 = 5,       /* Algorithm gives out 5 images */
    SEB_GET_6_OUT_IMAGES     = 6,       /* Algorithm gives out 6 images */
	SEB_GET_7_OUT_IMAGES 	 = 7,       /* Algorithm gives out 7 images */
    SEB_GET_8_OUT_IMAGES     = 8,       /* Algorithm gives out 8 images */
    SEB_GET_9_OUT_IMAGES     = 9,       /* Algorithm gives out 9 images */
    SEB_GET_10_OUT_IMAGES    = 10,       /* Algorithm gives out 10 images */
    
    /*Modes valid only for SEB_IP_SELECTION_T = SEB_LOW_LIGHT_MODE*/
    SEB_MFNR_MODE			 = 100,
    SEB_MFLE_MODE			 = 101,
    SEB_MFNR_MODE_SINGLE_RUN = 102,
    SEB_MFLE_MODE_SINGLE_RUN = 103,		/* Not currently used */
    
    SEB_OUT_MODE_UNKNOWN = 200      /* Will return SEB_INVALID_IN_PARAMS if used */
}SEB_OUT_MODE_T;

/*****************************************************************************/
/* Data structure definitions                                                */
/*****************************************************************************/
typedef struct SEB_point_t
{
	float f_pos_x;				/* co-ordinate in horizontal direction */
	float f_pos_y;				/* co-ordinate in vertical direction */
} seb_point_t;					/* structure for a point */

typedef struct SEB_metering_pts_t
{
	WORD32 i4_num_dark_points;
	WORD32 i4_num_bright_points;
	seb_point_t s_auto_metering_pt;                     /* Default value is (0.5, 0.5)  */
	seb_point_t as_dark_metering_pt[SEB_MAX_NUM_PTS];	/* Metering point indicating dark region (bright image) */
	seb_point_t as_bright_metering_pt[SEB_MAX_NUM_PTS];	/* Metering point indicating bright region (dark image) */

} seb_metering_pts_t;						/* structure for metering points 		   */

typedef struct _low_light_prev_data
{
	WORD32 i4_output_mode;
	WORD32 i4_num_images;
	WORD32 i4_prev_metadata_size;
	void   *pv_prev_metadata;
}low_light_prev_data_t;

typedef struct seb_exposure_iso_t
{
	WORD32 i4_num_dark_images;
	WORD32 i4_num_bright_images;
	WORD32 i4_ISO_value_ref_image;                      /* Reference image is not the same as auto image */
	float f_exposure_time_ref_image;
	WORD32 ai4_ISO_value_dark_image [SEB_MAX_NUM_BRACKETS];
	float af_exposure_time_dark_image[SEB_MAX_NUM_BRACKETS];
	WORD32 ai4_ISO_value_bright_image[SEB_MAX_NUM_BRACKETS];
	float af_exposure_time_bright_image[SEB_MAX_NUM_BRACKETS];
	low_light_prev_data_t s_low_light_data;
} seb_exposure_iso_t;


typedef struct seb_point_info_t
{
	seb_point_t s_metering_point_t;
	WORD32 i4_ISO_value;
	float f_exposure_time;
}
seb_point_info_t;


typedef struct seb_value_summary
{
	WORD32 i4_num_dark_points;
	WORD32 i4_num_bright_points;
	seb_point_info_t as_dark_pt_list[SEB_MAX_NUM_PTS];
	seb_point_info_t as_bright_pt_list[SEB_MAX_NUM_PTS];
	seb_point_info_t s_auto_pt;
}seb_value_summary;

typedef struct seb_init_params_t
{
	WORD32	i4_max_width;					/* Maximum width possible */
	WORD32	i4_max_height;					/* Maximum height possible */
	WORD32	i4_max_stride;					/* Maximum stride possible */
	WORD32	i4_max_inp_images;
	WORD32	i4_image_format;				/* Image format YUV420 or JPEG	*/
	WORD32	i4_camera_phone;				/* Camera Phone Model  		*/
	WORD32	i4_bracketing_mode;				/* To choose between SMP and SEB */
	WORD32	i4_memory_available;			/* Dummy variable not used */

}seb_init_params_t;

typedef struct	seb_in_params_t
{
	WORD32 	i4_width;					/* Input width of given input image buffer */
	WORD32 	i4_height;					/* Input height of given input image buffer */
	WORD32 	i4_stride;					/* Input stride of given input image buffer */
	WORD32	i4_format;					/* Input format of given input image buffer */
	WORD32          i4_num_inp_images;
	SEB_OUT_MODE_T	i4_num_out_images;          /* To choose number of output images */
	SEB_OUT_MODE_T  i4_max_num_out_images;
	UWORD32	u4_iso_min;
	UWORD32	u4_iso_max;
	float	f_exposure_time_min;
	float	f_exposure_time_max;
	seb_value_summary s_points_summary;
	WORD32 *pi4_ISO_table;
	WORD32 i4_ISO_table_length;
}seb_in_params_t;						/* structure of input params */

typedef struct	seb_in_image_t
{
	WORD32	i4_num_buffers;							/* Number of buffers in image (3 for YUV)  */
	UWORD8	*pu1_bufs[SEB_MAX_NUM_IMAGE_COMPONENTS];/* Pointer to y cb cr buffers 	   		   */
	WORD32	i4_size[SEB_MAX_NUM_IMAGE_COMPONENTS];	/* Number of buffers in input image		   */
}seb_in_image_t;									/* Total size of YUV buffer       		   */

typedef struct	seb_in_buffers_t
{
	seb_in_image_t	as_in_image[SEB_MAX_NUM_IMAGES];
}seb_in_buffers_t;					/* structure of input buffers */

typedef struct seb_out_params_t
{
	WORD32 i4_error_code;					/*  Flag returning error code */
	seb_metering_pts_t s_metering_points;		/*  output structure with metering points */
	seb_exposure_iso_t s_exposure_iso;						/* Output structure with exposure and ISO */
} seb_out_params_t;					/* structure of output buffers */


typedef struct seb_callbacks_t
{
	void * (*seb_memory_allocate)(WORD32 i4_size);		/* Memory allocation function call back for IP */
	void   (*seb_memory_free)(void *pv_ptr);			/* Memory free function call back for IP */
	WORD32   (*seb_file_dump)(void *pv_ptr,				/* Memory file dump call back for IP(used for debugging) */
			WORD32 i4_size,
			char *pi1_file_name,
			WORD32 is_append_mode);
}seb_callbacks_t;

/*****************************************************************************/
/* Function prototypes														 */
/*****************************************************************************/
void*  seb_init (  seb_init_params_t *ps_params,		/* Initialisation call for IP  */
				   seb_callbacks_t 	*ps_callbacks,
				   WORD32 		*pi4_error_code);

WORD32 seb_get_bracketing (void  	*pv_handle,					/* Process call of selection of metering points  */
		seb_in_params_t	    *ps_in_params,
		seb_in_buffers_t 	*ps_in_image,
		seb_out_params_t    *ps_out_params);

WORD32 seb_refine_bracketing (void  	*pv_handle,					/* Process call of selection of metering points  */
		seb_in_params_t	    *ps_in_params,
		seb_in_buffers_t 	*ps_in_image,
		seb_out_params_t    *ps_out_params);

WORD32 seb_calc_next_exposure_iso(void *pv_handle,
		 seb_in_params_t *ps_in_params,	seb_in_buffers_t *ps_in_image,
		 seb_out_params_t *ps_out_params);

WORD32 seb_delete( void 	*pv_handle);				/* Delete call of IP  */
#endif /* SEB_INTERFACE_H_ */
